<?php
/**
 * User: nester_a
 * Date: 29.10.13
 */

namespace Gtm\Repository;

use LMS\Tools\Conference\Repositories;
use LMS\Libraries\Exception\Http;
use LMS\Libraries\Exception\Database;
use LMS\Libraries\Exception\Operation;
use LMS\Libraries\Date;

class Booking extends Repositories\Booking
{

    private $oauthData;

	/**
	 * @var Gtmbooking
	 */
	private $gtmBookingRepository;

	/**
	 * @var Remote\Booking
	 */
	private $gtmRemoteBookingRepository;


    public function __construct($options)
    {
        $this->oauthData = $options['oauthData'];
        parent::__construct($options);

		$this->gtmBookingRepository = new Gtmbooking($options);
		$this->gtmRemoteBookingRepository = new Remote\Booking($options);
    }



    /**
     * Function to get list of available conferences at current time
     *
     * @param int $courseID
     * @return mixed
     */
    public function getAvailable($courseID, $timeOffset, $withTeacherNames=false)
    {
        $join = $withTeacherNames ? ' LEFT JOIN #__users as b ON a.teacher_id = b.id' : '';
        $select = $withTeacherNames ? ', b.username as teacher_name' : '';


        $currentTimestamp = ( time());
		$timeEarlierOnHour = $currentTimestamp - 60 * 60;
        $query = "SELECT a.* $select, gtm.meeting_id AS meeting_id, gtm.join_url AS join_url
                    FROM #__lms_conference_period a
                        $join
                        LEFT JOIN #__lms_conference_gtm gtm ON gtm.conf_id=a.p_id
                        WHERE
                            a.course_id='" .$courseID."'
                        AND
                            a.from_time <= ".$timeEarlierOnHour ."
                        AND
                            a.to_time >= ".$currentTimestamp;

        $this->db->setQuery( $query );
        return $this->db->loadObjectList();
    }

    /**
     * @param $bookedConferenceID
     * @return mixed
     */
    public function getMeetingByConferenceID($bookedConferenceID)
    {
        $query = "SELECT *
                    FROM
                        #__lms_conference_gtm WHERE conf_id=$bookedConferenceID";
        $this->db->setQuery($query);
        return $this->db->loadObject();
    }

    /**
     * @param $uniqueMeetingIDs
     * @return mixed
     */
    public function getDeletedFromGTMService($uniqueMeetingIDs)
    {
        $query = "SELECT conf_id
                    FROM #__lms_conference_gtm
                    WHERE
                        unique_meeting_id NOT IN ('".implode('\',\'', $uniqueMeetingIDs)."')";
        $this->db->setQuery($query);
        return $this->db->loadObjectList();
    }

    /**
     * Return count of booked conferences filtered by student id and teacher id
     *
     * @param int $courseID
     * @param int $teacherID
     * @param int $studentID
     * @return int
     */
    public function count($courseID, $teacherID=false, $studentID=false, $filterOptions=array())
    {
        $additionalJoin = $studentID ? (" LEFT JOIN #__lms_conference_usr as b ON a.p_id=b.p_id") : '';
        $additionalJoin.= ' JOIN #__lms_conference_gtm as gtmc ON gtmc.conf_id = a.p_id';

        $additionalWhere = $teacherID ? " AND a.teacher_id = $teacherID " : '';
        $additionalWhere.= $studentID ? " AND a.p_id = b.p_id AND b.user_id = $studentID" : "";

		if(isset($filterOptions['history']) && !$filterOptions['history']){
			$additionalWhere .= " AND to_time >= UNIX_TIMESTAMP(NOW())";
		}

		if(isset($filterOptions['future']) && !$filterOptions['future']){
			$additionalWhere .= " AND from_time < UNIX_TIMESTAMP(NOW())";
		}

        $query = "SELECT COUNT(distinct a.p_id)
                        FROM #__lms_conference_period as a
                        $additionalJoin
                                LEFT JOIN #__users as c ON a.teacher_id = c.id
                        WHERE course_id='".$courseID."'" . $additionalWhere;

        $this->db->setQuery($query);
        return $this->db->LoadResult();
    }




    /**
     * Insert(create) new booked conference
     *
     * @param int $courseID
     * @param int $teacherID
     * @param string $fromTime
     * @param string $toTime
     * @param int $cPublic
     * @param string $pName
     * @param string $pDescription
     * @return mixed
     */
    public function save($data)
    {
        $meetingData = $this->gtmRemoteBookingRepository->save($data);
        if($meetingData === false){
			throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
        }

        $conferenceID = parent::save($data) ? $this->lastInsertedID() : false;
        if($conferenceID == false){
			throw new Database\Error('Error occurred while saving in LMS table');
        }

		$dataToBeSaved = $meetingData[0];
		$dataToBeSaved->conferenceID = $conferenceID;

        if(!$this->gtmBookingRepository->save((array)$dataToBeSaved)){
			throw new Database\Error('Error occurred while saving in GTM table');
		}
		return true;
    }

	/**
	 * Insert(create) new booked conference
	 *
	 * @param int $courseID
	 * @param int $teacherID
	 * @param string $fromTime
	 * @param string $toTime
	 * @param int $cPublic
	 * @param string $pName
	 * @param string $pDescription
	 * @return mixed
	 */
	public function saveMultiple($data)
	{
		foreach($data as $item){
			$meetingData = $this->gtmRemoteBookingRepository->save($item);
			if($meetingData == false){
				throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
			}

			$conferenceID = parent::save($item) ? $this->lastInsertedID() : false;
			if($conferenceID == false){
				return false;
			}

			$dataToBeSaved = $meetingData[0];
			$dataToBeSaved->conferenceID = $conferenceID;

			$this->gtmBookingRepository->save((array)$dataToBeSaved);
		}

		return true;
	}

    /**
     * Delete specified booked conferences
     *
     * @param array $bookedConferenceIDs
     * @return mixed
     */
    public function delete($where = array())
    {
        $bookedConferenceIDs = $where['p_id'];

        $conferenceDeletedFromGTM = array();

        foreach($bookedConferenceIDs as $conferenceID){
			$meeting = $this->getMeetingByConferenceID($conferenceID);
            if($this->gtmRemoteBookingRepository->delete($meeting->meeting_id) !== false){
                $conferenceDeletedFromGTM[] = $conferenceID;
            }else{
				$httpErrorOccurred = true;
            }
        }

		if(empty($conferenceDeletedFromGTM)){
			if(!empty($bookedConferenceIDs)){
				throw new Operation\Error(
					'Error occurred. Seems like conference(s) already has been deleted from GoToMeeting account. Please run Synchronize'
				);
			}
			return true;
		}

        if(!$this->gtmBookingRepository->delete(array('conf_id' => $conferenceDeletedFromGTM))){
			throw new Database\Error('Error occurred while deleting from GTM table');
		}

		if(!parent::delete(array('p_id' => $conferenceDeletedFromGTM))){
			throw new Database\Error('Error occurred while deleting from LMS table');
		}

		if(isset($httpErrorOccurred)){
			throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
		}

        return true;
    }

    /**
     * Update info for booked conference
     *
     * @param int $bookedConferenceID
     * @param int $teacherID
     * @param string $fromTime
     * @param string $toTime
     * @param int $cPublic
     * @param string $pName
     * @param string $pDescription
     * @return mixed
     */
    public function update($params, $where = array())
    {
		$meeting = $this->getMeetingByConferenceID($where['p_id']);
		if(is_null($meeting)){
			return false;
		}

        if($this->gtmRemoteBookingRepository->update($meeting, $params) === false){
			throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
		}

        return parent::update($params, $where);
    }





	protected function prepareFilter($query, &$filterOptions)
	{
		$query = parent::prepareFilter($query, $filterOptions);
		$query->innerJoin($this->gtmBookingRepository->getTable()->getTableName() . ' as gtmc ON gtmc.conf_id = booking.p_id');
		return $query;
	}


    /**
     * @param $oauthData
     */
    public function setOAuthData($oauthData)
    {
        $this->oauthData = $oauthData;
    }


}