<?php
namespace LMS\Tools\Homework\Repositories;

use LMS\Libraries\Base;
use LMS\Tools\Homework\Tables;

class HomeworkResults extends Base\Repository
{
	public function __construct($options = array())
	{
		parent::__construct($options);
		$this->table = new Tables\HomeworkResults($this->db);
	}

	protected function prepareFilter($query, &$filterOptions)
	{
		$alias = $this->table->getTableAlias();

		$conditions = array();

		if(isset($filterOptions['id']) && $filterOptions['id']){
			$conditions[] = $alias.".id = ".$this->db->quote($filterOptions['id']);
		}

		if(isset($filterOptions['courseId']) && $filterOptions['courseId']){
			$conditions[] = $alias.".course_id = ".$this->db->quote($filterOptions['courseId']);
		}

		if(isset($filterOptions['userId']) && $filterOptions['userId']){
			$conditions[] = $alias.".user_id = ".$this->db->quote($filterOptions['userId']);
		}

		if(isset($filterOptions['homeworkId']) && $filterOptions['homeworkId']){
			$conditions[] = $alias.".hw_id = ".$this->db->quote($filterOptions['homeworkId']);
		}

		if(isset($filterOptions['homeworkStatus']) && $filterOptions['homeworkStatus']){
			$conditions[] = $alias.".hw_status = ".$this->db->quote($filterOptions['homeworkStatus']);
		}

		$query->where($conditions);

		return parent::prepareFilter($query, $filterOptions); // TODO: Change the autogenerated stub
	}

	public function getListQuery($filterOptions)
	{
		$query = $this->db->getQuery(true);

		$query->select('u.id as user_id, u.name, u.username, u.email');
		$query->from($this->db->quoteName('#__users', 'u'));
		$query->where($this->db->quoteName('u.id').' = '.$this->db->quoteName('uig.user_id'));
		if(isset($filterOptions['userId']) && $filterOptions['userId']){
			$query->where($this->db->quoteName('u.id').' = '.$this->db->quote($filterOptions['userId']));
		}

		if(isset($filterOptions['groupId']) && $filterOptions['groupId']){
			$query->from($this->db->quoteName('#__lms_users_in_global_groups', 'uigg'));
			$query->where($this->db->quoteName('uigg.user_id').' = '.$this->db->quoteName('uig.user_id'));
			$query->where($this->db->quoteName('uigg.group_id').' = '.$this->db->quote($filterOptions['groupId']));
		}

		$query->from($this->db->quoteName('#__lms_users_in_groups', 'uig'));
		$query->where($this->db->quoteName('uig.course_id').' = '.$this->db->quote($filterOptions['courseId']));

		$query->select('hwr.id, hwr.course_id, hwr.hw_id, hwr.hw_status, hwr.hw_date, hwr.grade');
		$query->leftJoin(
			$this->db->quoteName('#__lms_homework_results', 'hwr')
			. ' ON '
			. $this->db->quoteName('uig.user_id') . ' = ' . $this->db->quoteName('hwr.user_id')
			. ' AND '
			. $this->db->quoteName('hwr.hw_id') . ' = ' . $this->db->quote($filterOptions['homeworkId'])
		);

		if(isset($filterOptions['status']) && $filterOptions['status'] == 1){
			$query->where($this->db->quoteName('hwr.hw_id').' IS NOT NULL');
		} else
		if(isset($filterOptions['status']) && $filterOptions['status'] == 2){
			$query->where($this->db->quoteName('hwr.hw_id').' IS NULL');
		}

		if(isset($filterOptions['list.ordering'])){
			$query->order($filterOptions['list.ordering'].' '.(isset($filterOptions['list.direction']) ? $filterOptions['list.direction'] : 'ASC' ));
		}

		return $query;
	}

}