<?php
 
namespace LMS\Tools\Users\Model;
 
defined('_JEXEC') or die;

use LMS\Libraries;
use LMS\Libraries\Repository;
use LMS\Libraries\User;
use LMS\Libraries\Base;
use LMS\Libraries\ServiceLocator;
use LMS\Tools\Users\Repositories as Rep;

class Students extends Base\Model
{	
	var $context = 'students';	
	
	var $modelDir = __DIR__;
		
	public function __construct()
	{		
		$this->repository = Libraries\ServiceLocator::get('student_rep');
		
		parent::__construct();
		
		$this->setState('fields','student.profile.id, student.profile.username, student.profile.name, student.profile.email, student.group_id');
	}		
	
	protected function loadFormData() 
	{
		$data = new \JObject();				
		$data->set('search', $this->getState('filter.search'));		

		return $data;
	}
	
	public function getItems() 
	{
		$items = parent::getItems();
		
		$config = ServiceLocator::get('config');
		if($config->get('use_global_groups')){
			$userGroupRep = Libraries\ServiceLocator::get('studentgroup_rep');
			$stuIds = array();			
			
			foreach($items AS $item) {
				$stuIds[] = $item->id;
			}
				
			$stuGroups = $userGroupRep->getByFilter(array(			
				'user_id' => $stuIds
			));		
			
			foreach($items AS &$item) {
				foreach($stuGroups AS $group) {
					if($item->id == $group->user_id) {
						if(!is_array($item->group_id)) {							
							$item->group_id = array();
						}
						
						$item->group_id[] = $group->group_id;
					}
				}			
			}
		}		
		
		return $items;
		
	}
	
	public function countStudentsInGroup($groupId, $selectedStudents, $exceptedStudents)
	{
		$where = '';	
		$groupWhere = array();
			
		if(in_array(0, $groupId)) {
			$groupWhere[] = 'student.studentgroup.group_id IS NULL';			
			array_splice($groupId, array_search('0', $groupId), 1);			
		}		
		if(!empty($groupId)) {
			$groupWhere[] = 'student.studentgroup.group_id IN ('.implode(',', $groupId).')';		
		}
		
		$strGroupWhere = '';
		
		if(count($groupWhere) == 2) {
			$strGroupWhere = '('.implode($groupWhere, ' OR ').')';
		} else {
			$strGroupWhere = implode($groupWhere);
		}
		
		$studentWhere = '';
		
		if(!empty($selectedStudents)) {
			$studentWhere .= 'student.user_id IN ('.implode(',', $selectedStudents).')';
		}
		
		if(!empty($exceptedStudents)) {	
			if(!empty($strGroupWhere)) {
				$strGroupWhere .= ' AND ';
			}
			$where = '('.$strGroupWhere.'student.user_id NOT IN ('.implode(',', $exceptedStudents).'))'.$studentWhere;
		} else {
			if(!empty($strGroupWhere) && !empty($studentWhere)) {
				$strGroupWhere .= ' OR ';
			}
			$where = $strGroupWhere.$studentWhere;
		}
				
		$rep = Libraries\ServiceLocator::get('student_rep');
		
		$res = $rep->select('COUNT(DISTINCT student.user_id)',
			array(
				'where'=>$where,
				'resultType'=>'result'
			)
		);
				
		return $res;	
	}	
	
	public function getStudentsNames($ids, $excludedIds)
	{
		$options = array(
				'fields'=>'DISTINCT student.profile.name',
				'resultType'=>'column'
		);
		
		if(empty($ids) && empty($excludedIds)) {
			$options['user_id'] = array();
		} else {
			if(!empty($ids)) {
				$options['user_id'] = $ids;
			}
			
			if(!empty($excludedIds)) {	
				$options['excluded_user_id'] = $excludedIds;
			}
		}
		
		$rep	= ServiceLocator::get('student_rep');
		
		$sudentsNames = $rep->getByFilter($options);		
		
		return $sudentsNames;
	}
}